Package: scmeth
Type: Package
Title: Functions to conduct quality control analysis in methylation
        data
Version: 1.33.0
Author: Divy Kangeyan <divyswar01@g.harvard.edu>
Maintainer: Divy Kangeyan <divyswar01@g.harvard.edu>
Depends: R (>= 3.5.0)
Imports: BiocGenerics, bsseq, AnnotationHub, Seqinfo, GenomicRanges,
        reshape2, stats, utils, BSgenome, DelayedArray (>= 0.5.15),
        annotatr, SummarizedExperiment (>= 1.5.6), GenomeInfoDb,
        Biostrings, DT, HDF5Array (>= 1.7.5)
Suggests: knitr, rmarkdown, BSgenome.Mmusculus.UCSC.mm10,
        BSgenome.Hsapiens.NCBI.GRCh38,
        TxDb.Hsapiens.UCSC.hg38.knownGene, org.Hs.eg.db, Biobase,
        ggplot2, ggthemes
Description: Functions to analyze methylation data can be found here.
        Some functions are relevant for single cell methylation data
        but most other functions can be used for any methylation data.
        Highlight of this workflow is the comprehensive quality control
        report.
biocViews: DNAMethylation, QualityControl, Preprocessing, SingleCell,
        ImmunoOncology
BugReports: https://github.com/aryeelab/scmeth/issues
License: GPL-2
RoxygenNote: 7.3.2
VignetteBuilder: knitr
Config/pak/sysreqs: cmake make libbz2-dev libicu-dev liblzma-dev
        libpng-dev libuv1-dev libxml2-dev libssl-dev libx11-dev
        xz-utils zlib1g-dev
Repository: https://bioc.r-universe.dev
Date/Publication: 2026-04-28 12:47:18 UTC
RemoteUrl: https://github.com/bioc/scmeth
RemoteRef: HEAD
RemoteSha: f2ec0ad6754a7ae0f7db4480cf06eacb2a3052b9
NeedsCompilation: no
Packaged: 2026-05-30 09:44:15 UTC; root
Built: R 4.6.0; ; 2026-05-30 14:21:36 UTC; unix

# Generated by roxygen2: do not edit by hand

export(bsConversionPlot)
export(chromosomeCoverage)
export(coverage)
export(cpgDensity)
export(cpgDiscretization)
export(downsample)
export(featureCoverage)
export(mbiasplot)
export(methylationDist)
export(readmetrics)
export(repMask)
export(report)
import(BSgenome)
import(DT)
import(GenomicRanges)
import(SummarizedExperiment)
importFrom(BiocGenerics,organism)
importFrom(Biostrings,DNAString)
importFrom(Biostrings,vmatchPattern)
importFrom(DelayedArray,colMeans)
importFrom(DelayedArray,colSums)
importFrom(DelayedArray,rowSums)
importFrom(GenomeInfoDb,seqlevelsStyle)
importFrom(HDF5Array,loadHDF5SummarizedExperiment)
importFrom(Seqinfo,seqnames)
importFrom(annotatr,annotate_regions)
importFrom(annotatr,build_annotations)
importFrom(annotatr,builtin_genomes)
importFrom(annotatr,summarize_annotations)
importFrom(bsseq,getCoverage)
importFrom(stats,aggregate)
importFrom(stats,dbinom)
importFrom(stats,sd)
importFrom(utils,read.csv)
importFrom(utils,read.table)

[![Build Status](https://travis-ci.org/aryeelab/scmeth.svg?branch=master)](https://travis-ci.org/aryeelab/scmeth)

# scmeth
**scmeth** is a quality Control analysis package for methylation data. See the package vignette for various functions in the package. 

#' Downsample analysis
#'
#'Downsample the CpG coverage matrix for saturation analysis
#'@param bs bsseq object
#'@param subSample number of CpGs to subsample
#'Default value is 1000000.
#'@param offset how many CpGs to offset when subsampling
#'Default value is set to be 50000, i.e. first 50000 CpGs will
#'be ignored in subsampling.
#'@param dsRates downsampling rate. i.e. the probabaility of sampling
#'a single CpG
#'default is list of probabilities ranging from 0.01 to 1
#'0.01 0.02 0.05 0.10 0.20 0.30 0.40 0.50 0.60 0.70 0.80 0.90
#'For more continuous saturation curve dsRates can be changed to add more
#'sampling rates
#'@return Data frame with the CpG coverage for each sample at each
#'sampling rate
#'@examples
#'directory <- system.file("extdata/bismark_data", package='scmeth')
#'bs <- HDF5Array::loadHDF5SummarizedExperiment(directory)
#'scmeth::downsample(bs)
#'@importFrom stats dbinom
#'@importFrom bsseq getCoverage
#'
#'@export


downsample <- function(bs, dsRates = c(0.01, 0.02, 0.05, seq(0.1, 0.9, 0.1)), subSample=1e6, offset=50000){
    nCpGs <- nrow(bs)

    if (nCpGs<(subSample + offset)){
        bs <- bs
        subSample <- nCpGs
    }else{
        bs <- bs[offset:(subSample + offset)]
    }


    covMatrix <- bsseq::getCoverage(bs)
    nSamples <- dim(covMatrix)[2]
    downSampleMatrix <- matrix(nrow=length(dsRates) + 1, ncol=nSamples)
    maxCov <- 20
    nonZeroProbMatrix <- matrix(nrow=(length(dsRates) + 1), ncol=maxCov)

    for (i in seq_len(length(dsRates))){
        nonZeroProbMatrix[i,] <- 1 - dbinom(0, seq_len(maxCov), dsRates[i])
    }

    nonZeroProbMatrix[(length(dsRates) + 1),] <- 1

    countMatrix <- sapply(seq_len(ncol(covMatrix)), function(i) {
        cv = as.vector(covMatrix[,i])
        cv[cv>maxCov ] <- maxCov
        tab <- table(cv)
        tab <- tab[names(tab)!="0"]
        x <- rep(0, maxCov)
        x[as.numeric(names(tab))] <- tab
        x
    })

    downSampleMatrix <- round(nonZeroProbMatrix %*% countMatrix)
    downSampleMatrix <- downSampleMatrix*(nCpGs/subSample)

    rownames(downSampleMatrix) <- c(dsRates, 1)
    return(downSampleMatrix)
}

#' CpG covergae in each chromosome
#'
#'Provides Coverage metrics for each sample by the chromosome
#'@param bs bsseq object
#'@return matrix of chromsome covergae with
#'column and rows indicating the samples and the chromosome respectively
#'@examples
#'directory <- system.file("extdata/bismark_data",package='scmeth')
#'bs <- HDF5Array::loadHDF5SummarizedExperiment(directory)
#'chromosomeCoverage(bs)
#'@importFrom bsseq getCoverage
#'@importFrom Seqinfo seqnames
#'@export


chromosomeCoverage <- function(bs) {
    bs <- GenomeInfoDb::keepStandardChromosomes(bs)
    covMatrix <- bsseq::getCoverage(bs)
    Granges <- GenomicRanges::granges(bs)
    standardChr <- Seqinfo::seqnames(Granges)
    chrCov <- by(covMatrix>0, standardChr, colSums)
    chrCov <- do.call("rbind", chrCov)
    return(chrCov)
}

#'scmeth: a package to conduct quality control analysis for methylation data.
#'Most functions can be applied to both bulk and single-cell methylation
#'while other functions are specific to single-cell methylation data.
#'scmeth is especially customized to use the output from the FireCloud
#'implementation of methylation pipeline to produce comprehensive
#'quality control report
#'
#' @docType package
#' @name scmeth

NULL

## ----eval=FALSE---------------------------------------------------------------
# if (!requireNamespace("BiocManager", quietly=TRUE))
#     install.packages("BiocManager")
# BiocManager::install("scmeth")

## ----warning=FALSE, message=FALSE---------------------------------------------
library(scmeth)

## ----eval=FALSE---------------------------------------------------------------
# directory <- system.file("extdata", "bismark_data", package='scmeth')
# bsObject <- HDF5Array::loadHDF5SummarizedExperiment(directory)

## ----eval=FALSE---------------------------------------------------------------
# report(bsObject, '~/Documents', Hsapiens, "hg38")

## ----warning=FALSE, message=FALSE, comment=FALSE------------------------------
directory <- system.file("extdata", "bismark_data", package='scmeth')
bsObject <- HDF5Array::loadHDF5SummarizedExperiment(directory)

## -----------------------------------------------------------------------------
scmeth::coverage(bsObject)

## ----fig.width=6, fig.height=3------------------------------------------------
readmetrics(bsObject)

## ----warning=FALSE, message=FALSE---------------------------------------------
library(BSgenome.Mmusculus.UCSC.mm10)
load(system.file("extdata", 'bsObject.rda', package='scmeth'))
repMask(bs, Mmusculus, "mm10")

## ----warning=FALSE------------------------------------------------------------
chromosomeCoverage(bsObject)

## ----warning=FALSE,message=FALSE----------------------------------------------
#library(annotatr)
featureList <- c('cpg_islands', 'cpg_shores', 'cpg_shelves')
DT::datatable(featureCoverage(bsObject, features=featureList, "hg38"))

## ----warning=FALSE, message=FALSE---------------------------------------------
library(BSgenome.Hsapiens.NCBI.GRCh38)
DT::datatable(cpgDensity(bsObject, Hsapiens, windowLength=1000, small=TRUE))

## ----warning=FALSE------------------------------------------------------------
DT::datatable(scmeth::downsample(bsObject))

## ----warning=FALSE, message=FALSE, fig.width=6, fig.height=6------------------

methylationBiasFile <- '2017-04-21_HG23KBCXY_2_AGGCAGAA_TATCTC_pe.M-bias.txt'
mbiasList <- mbiasplot(mbiasFiles=system.file("extdata", methylationBiasFile,
                                         package='scmeth'))

mbiasDf <- do.call(rbind.data.frame, mbiasList)
meanTable <- stats::aggregate(methylation ~ position + read, data=mbiasDf, FUN=mean)
sdTable <- stats::aggregate(methylation ~ position + read, data=mbiasDf, FUN=sd)
seTable <- stats::aggregate(methylation ~ position + read, data=mbiasDf, FUN=function(x){sd(x)/sqrt(length(x))})
sum_mt<-data.frame('position'=meanTable$position,'read'=meanTable$read,
                       'meth'=meanTable$methylation, 'sdMeth'=sdTable$methylation,
                       'seMeth'=seTable$methylation)

sum_mt$upperCI <- sum_mt$meth + (1.96*sum_mt$seMeth)
sum_mt$lowerCI <- sum_mt$meth - (1.96*sum_mt$seMeth)
sum_mt$read_rep <- paste(sum_mt$read, sum_mt$position, sep="_")

g <- ggplot2::ggplot(sum_mt)
g <- g + ggplot2::geom_line(ggplot2::aes_string(x='position', y='meth',
                                                colour='read'))
g <- g + ggplot2::geom_ribbon(ggplot2::aes_string(ymin = 'lowerCI',
                        ymax = 'upperCI', x='position', fill = 'read'),
                        alpha=0.4)
g <- g + ggplot2::ylim(0,100) + ggplot2::ggtitle('Mbias Plot')
g <- g + ggplot2::ylab('methylation')
g


## ----warning=FALSE, message=FALSE, fig.width=6, fig.height=3------------------
methylationDist(bsObject)

## ----warning=FALSE, message=FALSE, fig.width=4, fig.height=6------------------
bsConversionPlot(bsObject)

## ----warning=FALSE, message=FALSE---------------------------------------------
sessionInfo()

#'Provides Coverage metrics in the repeat masker region
#'@param bs bsseq object
#'@param organism scientific name of the organism of interest,
#'e.g. Mmusculus or Hsapiens
#'@param genome reference alignment, i.e. mm10 or hg38
#'@return Data frame with sample name and coverage in repeat masker regions
#'@examples
#'library(BSgenome.Mmusculus.UCSC.mm10)
#'library(AnnotationHub)
#'load(system.file("extdata", 'bsObject.rda', package='scmeth'))
#'repMask(bs, Mmusculus, 'mm10')
#'@importFrom BiocGenerics organism
#'@importFrom DelayedArray colSums
#'@importFrom bsseq getCoverage
#'@export


repMask <- function(bs, organism, genome){
    GenomeInfoDb::seqlevelsStyle(bs) <- "UCSC"
    hub <- AnnotationHub::AnnotationHub()
    repeatGr <- hub[[names(AnnotationHub::query(hub,
                        c("rmsk", BiocGenerics::organism(organism), genome)))]]
    rep <- GenomicRanges::countOverlaps(bs, repeatGr)>0
    cov <- bsseq::getCoverage(bs)
    covDf <- data.frame(coveredCpgs = DelayedArray::colSums(cov[!rep,]>=1))
    return(covDf)
}

#' Generates an inclusive report on methylation analysis
#'
#' This function uses most of the functions in this package to
#' generate a report for the user
#'@param bsObj bsseq object
#'@param outdirectory name of the output directory where the
#'report will be saved
#'@param organism scientific name of the organism of interest,
#'e.g. Mmusculus or Hsapiens
#'@param genome reference alignment, e.g. mm10 or hg38
#'the report will have graphics on read information
#'@param mbiasDir Optional argument to provide directory name
#'that has the mbias files or the list of mbias files
#'@param subSample number of CpGs to subsample
#'Default value is 1000000.
#'@param offset how many CpGs to offset when subsampling
#'Default value is set to be 50000, i.e. first 50000 CpGs will
#'be ignored in subsampling.
#'@param small Indicator for a small dataset, cpg density is calculated more
#'@return Report will be an html file
#'@examples
#'library(BSgenome.Hsapiens.NCBI.GRCh38)
#'directory <- system.file("extdata/bismark_data", package='scmeth')
#'bs <- HDF5Array::loadHDF5SummarizedExperiment(directory)
#'mbiasDirectory=system.file("extdata", package='scmeth')
#'outDir <- system.file(package='scmeth')
#'report(bs, outDir, Hsapiens, 'hg38', mbiasDir=mbiasDirectory, small=TRUE)
#'@importFrom HDF5Array loadHDF5SummarizedExperiment
#'@import DT
#'@import SummarizedExperiment
#'@export
#

report <- function(bsObj, outdirectory, organism, genome, mbiasDir=NULL,
                   subSample=1e6, offset=50000, small=FALSE) {

    RmdFile <- system.file(".", 'qcReport.Rmd', package="scmeth")
    rmarkdown::render(RmdFile, params=list(outdir=outdirectory, samples=bsObj,
                                        organism=organism, genome=genome,
                                        mbias=mbiasDir, nCpGs=subSample,
                                        offset=offset, small=small),
                     knit_root_dir=getwd(),
                    output_file=paste0(outdirectory, "/qcReport.html"))

}

#' Discretize the CpG methylation values
#' to align with single cell analysis
#'
#'In single cell analysis overwhelmingly large number of CpGs have binary
#'methylation
#'Due to errors in sequencing and amplification many CpGs tend to have
#'non-binary methylation. Hence
#'this function catergorizes the non-binary CpGs as methylated if the
#'methyation is above 0.8 and
#'unmethylated if the methylation is below 0.2
#'@param bs bsseq object
#'@param subSample number of CpGs to subsample.
#'Default value is 1000000.
#'@param offset how many CpGs to offset when subsampling
#'Default value is set to be 50000, i.e. first 50000 CpGs will
#'be ignored in subsampling.
#'@param coverageVec If coverage vector is already calculated provide it to
#'speed up the process
#'@return meth discretized methylation matrix
#'@return discard total number of removed CpGs from each sample
#'@return Percentage of CpGs discarded compared to the total number of CpGs
#'@examples
#'directory <- system.file("extdata/bismark_data", package='scmeth')
#'bs <- HDF5Array::loadHDF5SummarizedExperiment(directory)
#'cpgDiscretization(bs)
#'@importFrom DelayedArray colSums
#'@importFrom bsseq getCoverage
#'@export


cpgDiscretization <- function(bs, subSample=1e6, offset=50000, coverageVec=NULL){
    # subsampling
    nCpGs <- nrow(bs)


    if (subSample == 'all'){
      bs <- bs
    }else{
      if (nCpGs < (subSample + offset)){
        bs <- bs
        subSample <- nCpGs
      }else{
        bs <- bs[offset:(subSample + offset)]
      }
    }


    covMatrix <- bsseq::getCoverage(bs)
    methMatrix <- bsseq::getCoverage(bs, type='M')
    nSamples <- ncol(methMatrix)
    methMatrix <- methMatrix/covMatrix
    if (is.null(coverageVec)){
        covVec <- DelayedArray::colSums(covMatrix>0, na.rm=TRUE)
        covVec <- covVec*(nCpGs/subSample)
    }else{
        covVec <- coverageVec
    }

    # Only consider methylation between 0.2 and 0.8
    methCutOff <- c(0.01, 0.19, 0.79, 1.0)

    methylationDistMatrix <- sapply(seq_len(nSamples), function(i) {
        mv = as.vector(methMatrix[,i])
        mv <- mv[!is.na(mv)]
        mvBin <- cut(mv, methCutOff)
        tab <- table(mvBin)
        x <- tab
        x
    })

    removedCpGs <- methylationDistMatrix[2,]*(nCpGs/subSample)
    removedCpGFrac <- (removedCpGs/(covVec))*100
    returnList <- list('discard' = removedCpGs,
                        'discardPerc' = removedCpGFrac)
    return(returnList)
}

#'Provides Coverage by the CpG density. CpG Density is defined as the number
#'of CpGs observed in certain base pair long region.
#'@param bs bsseq object
#'@param organism scientific name of the organism of interest,
#'e.g. Mmusculus or Hsapiens
#'@param windowLength Length of the window to calculate the density
#'@param small Indicator for a small dataset, cpg density is calculated more
#'memory efficiently for large dataset but for small dataset a different quicker
#'method is used
#'Default value for window length is 1000 basepairs.
#'@return Data frame with sample name and coverage in repeat masker regions
#'@examples
#'library(BSgenome.Hsapiens.NCBI.GRCh38)
#'directory <- system.file("extdata/bismark_data", package='scmeth')
#'bs <- HDF5Array::loadHDF5SummarizedExperiment(directory)
#'cpgDensity(bs, Hsapiens, 1000, small=TRUE)
#'@import BSgenome
#'@importFrom bsseq getCoverage
#'@importFrom Biostrings DNAString
#'@importFrom Biostrings vmatchPattern
#'@export

cpgDensity <- function(bs, organism, windowLength=1000, small=FALSE){

    #GenomeInfoDb::seqlevelsStyle(gr) <- GenomeInfoDb::seqlevelsStyle(organism)[1]
    cov <- bsseq::getCoverage(bs)
    gr <- GenomicRanges::granges(bs)
    cpg <- Biostrings::DNAString("CG")

    if (!small){
      cpg_gr <- Biostrings::vmatchPattern(cpg, organism)
      cpg_gr <- GenomeInfoDb::keepStandardChromosomes(cpg_gr, pruning.mode= "coarse")

      r_cpg_gr <- GenomicRanges::resize(cpg_gr, width=(windowLength/2), fix='center')
      cpgd <- GenomicRanges::countOverlaps(gr, r_cpg_gr)

    }else{
      gr_resized <- GenomicRanges::resize(gr, width=(500/2),fix='center')
      v <- Biostrings::Views(organism, gr_resized)
      dnFrequency <- Biostrings::dinucleotideFrequency(v)
      cpgd <- dnFrequency[,'CG']
    }


    #cpgd <- Repitools::cpgDensityCalc(gr, organism, window = windowLength)

    maxcpgd <- max(cpgd)
    cpgdCov <- sapply(seq_len(ncol(cov)), function(i) {
        cv = as.vector(cov[,i])
        cpgdCell <- cpgd[cv>0 ]
        tab <- table(cpgdCell)
        x <- rep(0, maxcpgd)
        x[as.numeric(names(tab))] <- tab
        x
    })

    rownames(cpgdCov) <- seq_len(maxcpgd)
    return(cpgdCov)
}

#' Provide graphics for read information
#'
#'Plot the mapped and unmapped reads
#'@param bs bsseq object
#'@return Plot showing the mapped and unmapped read information for each cell
#'@examples
#'directory <- system.file("extdata/bismark_data", package='scmeth')
#'bs <- HDF5Array::loadHDF5SummarizedExperiment(directory)
#'readmetrics(bs)
#'@export

readmetrics <- function(bs){

    phenotypicData <- Biobase::pData(bs)
    dat <- data.frame(sample=rownames(phenotypicData),
                    total=as.vector(phenotypicData$total_reads),
                    mapped=as.vector(phenotypicData$uniquely_aligned_reads +
                                phenotypicData$non_uniquely_aligned_reads))
    dat$unmapped <- dat$total - dat$mapped
    return(dat)
}

#' Provide graphics for methylation distribution
#'
#'Plot the methylation distribution for the cells in bsseq object
#'@param bs bsseq object
#'@return mean methylation for each sample
#'@examples
#'directory <- system.file("extdata/bismark_data", package='scmeth')
#'bs <- HDF5Array::loadHDF5SummarizedExperiment(directory)
#'methylationDist(bs)
#'@importFrom bsseq getCoverage
#'@importFrom DelayedArray colMeans
#'@export


methylationDist <- function(bs){
    covMatrix <- bsseq::getCoverage(bs)
    methMatrix <- bsseq::getCoverage(bs, type='M')/covMatrix
    methVec <- DelayedArray::colMeans(methMatrix, na.rm=TRUE)
    return(methVec)
}

#' Coverage for single cells
#'
#'Provides Coverage for each cell in a library pool
#'@param bs bsseq object
#'@param subSample number of CpGs to subsample.
#'Default value is 1000000.
#'@param offset how many CpGs to offset when subsampling
#'Default value is set to be 50000, i.e. first 50000 CpGs will
#'be ignored in subsampling.
#'@return vector of coverage for the cells in bs object
#'@examples
#'directory <- system.file("extdata/bismark_data", package='scmeth')
#'bs <- HDF5Array::loadHDF5SummarizedExperiment(directory)
#'coverage(bs)
#'@importFrom DelayedArray colSums
#'@importFrom bsseq getCoverage
#'@export


coverage <- function(bs, subSample=1e6, offset=50000) {
    nCpGs <- nrow(bs)

    if (subSample == 'all'){
        bs <- bs
        ratio <- 1
    }else{
        if (nCpGs < (subSample + offset)){
            bs <- bs
            subSample <- nCpGs
            ratio <- 1
        }else{
            bs <- bs[offset:(subSample + offset)]
            ratio <- nCpGs/subSample
        }
    }
    covMatrix <- bsseq::getCoverage(bs)
    covVec <- DelayedArray::colSums(covMatrix>0, na.rm=TRUE)
    covVec <- covVec*ratio
    return(covVec)
}

#' Coverage based on the genomic feature
#'
#'Provides Coverage metrics for the sample by each genomic features provided
#'by the user
#'@param bs bsseq object
#'@param features list of genomic features, e.g. genes_exons, genes_introns,
#'cpg_islands, cpg_shelves
#'Names are based on the annotatr packages, so all the features provided by the
#'annotatr
#'package will be supported in this function
#'@param genomebuild reference alignment, i.e. mm10 or hg38
#'@return a data frame with genomic feature names and the number of
#'CpG covered in each feature
#'@examples
#'directory <- system.file("extdata/bismark_data", package='scmeth')
#'bs <- HDF5Array::loadHDF5SummarizedExperiment(directory)
#'featureCoverage(bs, c('cpg_islands', 'cpg_shores'), 'hg38')
#'@importFrom DelayedArray rowSums
#'@importFrom GenomeInfoDb seqlevelsStyle
#'@importFrom annotatr builtin_genomes
#'@importFrom annotatr build_annotations
#'@importFrom annotatr annotate_regions
#'@importFrom annotatr summarize_annotations
#'@import GenomicRanges
#'@export


featureCoverage <- function(bs, features, genomebuild){

    annotationFeatures <- c()
    for (i in features){
    annotationFeatures <- c(paste0(genomebuild, '_', i), annotationFeatures)
    }

    annots_gr = annotatr::build_annotations(genome = genomebuild,
                                        annotations = annotationFeatures)
    GenomeInfoDb::seqlevelsStyle(bs) <- "UCSC"
    nSamples <- dim(bs)[2]

    sumAnnotMatrix <- matrix(nrow=length(features), ncol=nSamples)
    featureLabel <- rep(NA, length(features))
    for (i in seq_len(nSamples)){
        bsCell <- bs[,i]

        # CpGs that are observed
        coverageMatrix <- getCoverage(bsCell)
        ind <- DelayedArray::rowSums(coverageMatrix)>0
        # Intersect the regions with the reference annotations

        dm_annotated = annotatr::annotate_regions(
            regions = GenomicRanges::granges(bsCell)[ind,],
            annotations = annots_gr,
            ignore.strand = TRUE,
            quiet = TRUE)
        sumAnnot <- annotatr::summarize_annotations(dm_annotated, quiet=TRUE)
        sumAnnotMatrix[,i] <- sumAnnot$n[match(annotationFeatures, sumAnnot$annot.type)]/sum(ind)
        featureLabel[1:length(sumAnnot$annot.type)] <- sumAnnot$annot.type
    }
    colnames(sumAnnotMatrix) <- colnames(bs)
    rownames(sumAnnotMatrix) <- features

    return(sumAnnotMatrix)
}

#' Methylation bias plot
#'
#'Plot the methylation at each position of the read to observe any biases in
#'the methylation
#'based on the read position
#'
#'@param dir directory name with mbias files
#'@param mbiasFiles list of mbias files
#'@return Returns a list containing the methylation across the read position in original top
#'and original bottom strand both in forward and reverse reads for multiple samples
#'@examples
#'mbiasFile <- '2017-04-21_HG23KBCXY_2_AGGCAGAA_TATCTC_pe.M-bias.txt'
#'mbiasplot(mbiasFiles=system.file("extdata", mbiasFile, package='scmeth'))
#'@importFrom utils read.table
#'@importFrom utils read.csv
#'@importFrom stats sd
#'@importFrom stats aggregate
#'@export

mbiasplot <- function(dir=NULL, mbiasFiles=NULL){
    if (!is.null(dir)){
        mbiasFileList <- list.files(dir, pattern="*.M-bias.txt", full.names=TRUE)

    }else{
        mbiasFileList <- mbiasFiles
    }

    mbiasTableList <- list()
    nSamples <- length(mbiasFileList)
    for (i in seq_len(nSamples)){
        MbiasFile <- readLines(mbiasFileList[i])
        nvec <- length(MbiasFile)
        breaks <- which(!nzchar(MbiasFile))
        nbreaks <- length(breaks)
        if (breaks[nbreaks] < nvec) {
            breaks <- c(breaks, nvec + 1L)
            nbreaks <- nbreaks + 1L
        }

    if (nbreaks > 0L) {
        oracle <- mapply(function(a, b) paste(MbiasFile[a:b]),
                        c(1L, 1L + breaks[-nbreaks]) + 2,
                        breaks - 1L, SIMPLIFY = FALSE)
    }

    CpG_Mbias_Read1 <- utils::read.csv(sep='\t', text=oracle[[1]])
    CpG_Mbias_Read1$read <- 'read-1'
    CpG_Mbias_Read1$methylation <- CpG_Mbias_Read1$count.methylated*100/
      (CpG_Mbias_Read1$count.methylated + CpG_Mbias_Read1$count.unmethylated)
    CpG_Mbias_Read2 <- utils::read.csv(sep='\t', text=oracle[[4]])
    CpG_Mbias_Read2$read <- 'read-2'
    CpG_Mbias_Read2$methylation <- CpG_Mbias_Read2$count.methylated*100/
      (CpG_Mbias_Read2$count.methylated + CpG_Mbias_Read2$count.unmethylated)


    mbiasTable <- rbind(CpG_Mbias_Read1[, c('position', 'methylation', 'read')],
                        CpG_Mbias_Read2[, c('position', 'methylation', 'read')])
    mbiasTableList[[i]] <- mbiasTable
    }

    return(mbiasTableList)
}

#' Bisulfite conversion rate visualization
#'
#'Plot the bisulfite conversion rate for each sample
#'based on the pheno data in the bs object
#'@param bs bsseq object
#'@return Plot showing bisulfite conversion rate for each sample
#'@examples
#'directory <- system.file("extdata/bismark_data", package='scmeth')
#'bs <- HDF5Array::loadHDF5SummarizedExperiment(directory)
#'bsConversionPlot(bs)
#'@export


bsConversionPlot <- function(bs){
    phenoData <- bsseq::pData(bs)
    phenoData$bsconversion <- 1 - (phenoData$CHH_meth + phenoData$CHG_meth)/
                                    (phenoData$CHH_meth + phenoData$CHH_unmeth+
                                    phenoData$CHG_meth + phenoData$CHG_unmeth)
    bscDf <- data.frame(sample=rownames(phenoData), bsc=phenoData$bsconversion)
    return(bscDf)
}

---
title: "QC Report"
author: 
date: 
output: 
    html_document:
        smart: false      
params:
  samples: bs
  outdir: outdirectory
  organism: organism
  genome: genome
  mbias: mbiasDir
  nCpGs: subSample
  offset: offset
  small: small
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE)
```

## Rendered by scmeth

```{r, echo=FALSE, warning=FALSE, message=FALSE}
options(DelayedArray.block.size=4500e6)
bs <- params$samples
nSamples <- length(Biobase::sampleNames(bs))
phenoData <- Biobase::pData(bs)

```

## Read information
```{r read_metrics, eval=!is.null(phenoData$total_reads), echo=FALSE, warning=FALSE, message=FALSE, fig.width=8, fig.height=7, fig.align='center'}
# Methylation distribution for all the cells
# Check the commits
dat <- scmeth::readmetrics(bs)

SummaryTable <- dat
SummaryTable$sample <- sub('\\..*$', '', SummaryTable$sample)
o <- order(dat$total, dat$sample)
sampleOrder <- dat$sample[o]
Summary_read <- summary(dat[,c('total', 'mapped', 'unmapped')])





m <- reshape2::melt(dat[,c("sample", "mapped", "unmapped")], id.vars="sample",
                    variable.name="Mapping_status")
m$sample <- factor(m$sample, levels=sampleOrder)
m$Mapping_status <- relevel(m$Mapping_status, ref="unmapped")
g <- ggplot2::ggplot(m, ggplot2::aes_string('sample', 'value', 
                                           fill='Mapping_status'))
g <- g + ggplot2::geom_bar(stat="identity") + ggplot2::coord_flip()
g <- g + ggplot2::scale_y_continuous(name="Number of reads")
g <- g + ggplot2::xlab("samples")
g <- g + ggplot2::ggtitle("Read mapping stats")
g <- g + ggplot2::theme(panel.background = 
                        ggplot2::element_rect(fill = "white", colour = "grey50"), 
                        axis.text.y=ggplot2::element_blank(),
                        axis.ticks=ggplot2::element_blank())
g
```

```{r read_metrics_null, eval=is.null(phenoData$total_reads), echo=FALSE, warning=FALSE, message=FALSE, fig.width=8, fig.height=7, fig.align='center'}
print("Read information not provided in the phenotypic data")
SummaryTable <- data.frame('sample'=Biobase::sampleNames(bs))

```



```{r mbias, echo=FALSE, warning=FALSE, message=FALSE, eval=!is.null(params$mbias), fig.width=8, fig.height=7, fig.align='center'}
## Methylation bias plot
# Methylation distribution for all the cells
mbiasTableList <- scmeth::mbiasplot(dir=params$mbias)
mbiasDf <- do.call(rbind.data.frame, mbiasTableList)
mbiasDf$methylation <- round(mbiasDf$methylation, 4)
write.table(mbiasDf, paste0(params$outdir, '/mbias_Table.txt'), sep = "\t", 
quote=FALSE, row.names=FALSE)

meanTable <- stats::aggregate(methylation ~ position + read, data=mbiasDf, FUN=mean)
sdTable <- stats::aggregate(methylation ~ position + read, data=mbiasDf, FUN=sd)
seTable <- stats::aggregate(methylation ~ position + read, data=mbiasDf, FUN=function(x){sd(x)/sqrt(length(x))})
sum_mt <- data.frame('position'=meanTable$position, 'read'=meanTable$read,
                       'meth'=meanTable$methylation, 'sdMeth'=sdTable$methylation,
                       'seMeth'=seTable$methylation)
#sum_mt <- mt %>% dplyr::group_by(position,read) %>%
                        #dplyr::summarise(meth = mean(X..methylation),
                           #     sdMeth=stats::sd(X..methylation))
#sum_mt$seMeth <- sum_mt$sdMeth/sqrt(nSamples)
sum_mt$upperCI <- sum_mt$meth + (1.96*sum_mt$seMeth)
sum_mt$lowerCI <- sum_mt$meth - (1.96*sum_mt$seMeth)
sum_mt$read_rep <- paste(sum_mt$read, sum_mt$position, sep="_")

g <- ggplot2::ggplot(sum_mt)
g <- g + ggplot2::geom_line(ggplot2::aes_string(x='position', y='meth',
                                                colour='read'))
g <- g + ggplot2::geom_ribbon(ggplot2::aes_string(ymin = 'lowerCI',
                        ymax = 'upperCI', x='position', fill = 'read'),
                        alpha=0.4)
g <- g + ggplot2::ylim(0, 100) + ggplot2::ggtitle('Mbias Plot')
g <- g + ggplot2::ylab('methylation')
g <- g + ggplot2::theme_bw()
g
```


## CpG Coverage 
```{r cpg_coverage, echo=FALSE, warning=FALSE, message=FALSE, fig.height=5, fig.width=5, fig.align='center'}

# Coverage
covVec <- scmeth::coverage(bs, subSample=params$nCpGs, offset=params$offset)
sampleNames <- bsseq::sampleNames(bs)
covDf <- data.frame(sample=sampleNames, coveredCpgs=covVec)
o <- order(covDf$coveredCpgs)
sampleOrder <- covDf$sample[o]
covDf$sample <- factor(covDf$sample, levels=sampleOrder)

SummaryTable$CpG.coverage <- round(covVec)

g <- ggplot2::ggplot(covDf, ggplot2::aes_string(x="''", y='coveredCpgs'))
g <- g + ggplot2::geom_boxplot()
g <- g + ggplot2::geom_jitter() 
g <- g + ggplot2::xlab("")+ggplot2::ylab("Covered CpGs")
g <- g + ggplot2::ggtitle("Number of Covered CpGs")
g <- g + ggplot2::theme_bw()
g

#### Coverage Summary #####

summary(SummaryTable$CpG.coverage)

#############################
```

## Bisulfite conversion rate
```{r bs_conversion, echo=FALSE, warning=FALSE, eval=!is.null(phenoData$CHG_meth), message=FALSE, fig.height=5, fig.width=5, fig.align='center'}

bscDf <- scmeth::bsConversionPlot(bs)


SummaryTable$Bisulfite.converion.Rate <- round(bscDf$bsc, 4)

g <- ggplot2::ggplot(bscDf, ggplot2::aes_string(x="''", y='bsc'))
g <- g + ggplot2::geom_boxplot()
g <- g + ggplot2::ylim(max(min(bscDf$bsc) - 0.02, 0), min(max(bscDf$bsc) + 0.02 ,1))
g <- g + ggplot2::theme_bw()
g <- g + ggplot2::geom_jitter()
g <- g + ggplot2::xlab('') + ggplot2::ylab('bisulfite conversion rate')
g <- g + ggplot2::ggtitle('Bisulfite conversion rate across samples')
g
```


```{r bs_conversion_2, echo=FALSE, warning=FALSE, eval=is.null(phenoData$CHG_meth), message=FALSE, fig.height=5, fig.width=5, fig.align='center'}

print("bisulfite conversion rate information not provided in the phenotypic data")

```


## Methylation Density plot
```{r meth_density, echo=FALSE, warning=FALSE, message=FALSE, fig.height=5, fig.width=5, eval=TRUE, fig.align='center'}
cpgDenMatrix <- scmeth::cpgDensity(bs, organism = params$organism, 
                                   windowLength=1000, small=params$small)
cpgDenMatrix <- apply(cpgDenMatrix, 1, function(x) x/colSums(cpgDenMatrix))
m <- reshape2::melt(cpgDenMatrix, varnames = c("Sample", "Density"))


g <- ggplot2::ggplot(m, ggplot2::aes_string('Density', 'value'))
g <- g + ggplot2::geom_line(ggplot2::aes_string(group='Sample'))
g <- g + ggplot2::ggtitle("Fraction of covered CpGs by CpG density") 
g <- g + ggplot2::xlab("CpG coverage based on the CpG density") 
g <- g + ggplot2::xlab('CpG Density') + ggplot2::ylab('Fraction of CpGs')
g <- g + ggplot2::theme(axis.text.y = ggplot2::element_text(angle = 60, hjust = 1))
g <- g + ggplot2::theme_bw()
g
```



```{r non_rm_coverage, echo=FALSE, warning=FALSE, message=FALSE, fig.height=10, eval=FALSE, fig.align='center'}
## Non-repatmasker Coverage
covDf <- scmeth::repMask(bs, params$organism, params$genome)
covDf$sample <- rownames(covDf)
m <- reshape2::melt(covDf, id.vars = c("sample", "coveredCpgs"))
o <- order(covDf$coveredCpgs)
sampleOrder <- covDf$sample[o]
m$sample <- factor(m$sample, levels=sampleOrder)

g <- ggplot2::ggplot(m, ggplot2::aes_string('sample', 'coveredCpgs'))
g <- g + ggplot2::geom_bar(stat="identity") + ggplot2::coord_flip()
g <- g + ggplot2::scale_y_continuous(name="Number of CpGs") 
g <- g + ggplot2::xlab("Sample") 
g <- g + ggplot2::ggtitle("Number of Covered Non-repeatmasker CpGs")  
g <- g + ggplot2::theme_bw()
g
```

# Data Preprocessing
## CpG Discretization

```{r cpg_discretization, echo=FALSE, warning=FALSE, message=FALSE, fig.height=5, fig.width=5, eval=TRUE, fig.align='center'}
# CpG Discretization
discretizedCpG <- cpgDiscretization(bs, subSample=params$nCpGs, offset=params$offset, coverageVec=covVec)

SummaryTable$CpG.nonBinary.percentage <- round(discretizedCpG$discardPerc, 4)


#percentDiscarded <- discretizedCpG[3]
percentDiscarded <- discretizedCpG[2]
sampleNames <- bsseq::sampleNames(bs)
discardedDf <- data.frame(sample=sampleNames, value=percentDiscarded)
meltedDiscardedDf <- reshape2::melt(discardedDf, 
                                   id.vars = c("sample", "discardPerc"))

o <- order(meltedDiscardedDf$discardPerc, meltedDiscardedDf$sample)
sampleOrder <- meltedDiscardedDf$sample[o]
meltedDiscardedDf$sample <- factor(meltedDiscardedDf$sample, levels=sampleOrder)

g <- ggplot2::ggplot(meltedDiscardedDf, ggplot2::aes_string(x="''", y='discardPerc'))
g <- g + ggplot2::geom_boxplot()
g <- g + ggplot2::geom_jitter()
g <- g + ggplot2::xlab("") + ggplot2::ylab("Percentage of Discarded CpGs")
g <- g + ggplot2::ggtitle("Percentage of Discarded CpGs")
g <- g + ggplot2::theme_bw()
g
```

## Feature level coverage
```{r feature_coverage, echo=FALSE, warning=FALSE, message=FALSE, fig.height=5, fig.width=5, eval=TRUE , fig.align='center'}
#library(annotatr)
featureCovMatrix <- scmeth::featureCoverage(bs, c('genes_promoters', 'genes_exons', 'genes_introns', 'genes_intergenic', 'cpg_islands', 'cpg_shelves', 'cpg_shores', 'cpg_inter'), params$genome)

featureCovDf <- reshape2::melt(featureCovMatrix)

g <- ggplot2::ggplot(featureCovDf, ggplot2::aes_string(x="Var1", y='value'))
g <- g + ggplot2::geom_boxplot()+ggplot2::ylim(0, 1)
g <- g + ggplot2::geom_jitter()
g <- g + ggplot2::xlab("Features") + ggplot2::ylab("Fraction of CpGs")
g <- g + ggplot2::ggtitle("Feature Coverage Distribution")
g <- g + ggplot2::theme(axis.text.x = ggplot2::element_text(angle = 60, hjust=1))
g <- g + ggplot2::theme(panel.background = ggplot2::element_blank())
g

```

## Downsampling Analysis
```{r downsampling, echo=FALSE, warning=FALSE, message=FALSE, fig.height=7, fig.width=10, eval=TRUE, fig.align='center'}

##############################
# Downsample Plots
dsMatrix <- scmeth::downsample(bs, subSample=params$nCpGs, offset=params$offset)
colnames(dsMatrix) <- SummaryTable$sample
write.table(t(dsMatrix), paste0(params$outdir, '/Downsample_Table.txt'), sep = "\t", quote=FALSE)

meltedDsMatrix <- reshape2::melt(dsMatrix)
g <- ggplot2::ggplot(meltedDsMatrix, ggplot2::aes_string(x='Var1', y='value', group='Var2'))
g <- g + ggplot2::geom_line(alpha=0.2) 
g <- g + ggplot2::scale_x_continuous(name="Fraction of Reads")
g <- g + ggplot2::scale_y_continuous(name="Number of CpGs covered") 
g <- g + ggplot2::ggtitle("Saturation Analysis Plot") 
g <- g + ggplot2::stat_summary(ggplot2::aes_string(y='value', group='Var1'), fun.y=median, lwd=2, geom="point", alpha=1.0, color='green')
g <- g + ggplot2::theme_bw(base_size=11)
g

```

## Mean Methylation
```{r meth_dist, echo=FALSE, warning=FALSE, message=FALSE, fig.height=7, fig.width=10, eval=TRUE, fig.align='center'}
# Methylation distribution for all the cells
methylVec <- scmeth::methylationDist(bs)
methylDf <- data.frame('sample'=sampleNames, 'meth'=methylVec)

SummaryTable$meanMeth <- round(methylVec, 4)
if (!is.null(phenoData$total_reads)){
  colnames(SummaryTable) <- c('sample.name', 'total.reads', 'mapped.reads', 'unmapped.reads', 'CpG.coverage', 'Bisulfite.conversion.rate', 'CpG.nonBinary.percentage', 
'mean.methylation')
} else {
  colnames(SummaryTable) <- c('sample.name', 'CpG.coverage', 'CpG.nonBinary.percentage', 'mean.methylation')
}
  
  
write.table(SummaryTable, paste0(params$outdir, '/QC_Summary.txt'), sep = "\t", quote=FALSE, row.names=FALSE)

o <- order(methylDf$meth, methylDf$sample)
sampleOrder <- methylDf$sample[o]

methylDf$sample <- factor(methylDf$sample, levels=sampleOrder)

g <- ggplot2::ggplot(methylDf, ggplot2::aes_string('sample', 'meth'))
g <- g + ggplot2::geom_bar(stat="identity") + ggplot2::coord_flip()
g <- g + ggplot2::scale_y_continuous(name="Mean Methylation")
g <- g + ggplot2::xlab("samples")
g <- g + ggplot2::theme(panel.background = 
                        ggplot2::element_rect(fill = "white", colour = "grey50"), 
                        axis.text.y=ggplot2::element_blank(),
                        axis.ticks=ggplot2::element_blank())
g

#### Methylation Summary #####

summary(methylDf$meth)
```

---
title: "scmeth Vignette"
author: "Divy S. Kangeyan"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
    %\VignetteIndexEntry{Vignette Title}
    %\VignetteEngine{knitr::rmarkdown}
    %\VignetteEncoding{UTF-8}
---
Contents
-----------
1. Introduction 

2. Installation

3. Input files

4. Usage

    4.1 Report 
    
    4.2 Functions
    
---------------------

    
1. Introduction
---------------------
<p style="text-align: justify;">
Though a small chemical change in the genome, DNA methylation has significant
impact in several diseases, developmental processes and other biological 
changes. Hence methylation data should be analyzed carefully to gain 
biological insights. **scmeth** package offers a few functions to assess
the quality of the methylation data. 
</p>

<p style="text-align: justify;">
This bioconductor package contains functions to perform quality control and 
preprocessing analysis for single-cell methylation data. *scmeth* is 
especially customized to be used with the output from the FireCloud 
implementation of methylation pipeline. In addition to individual functions,
**report** function in the package provides all inclusive report using most of
the functions. If users prefer they can just use the **report** function to 
gain summary of their data.
</p>

2. Installation and package loading
------------------------------------
**scmeth** is available in bioconductor and can be downloaded using the 
following commands
```{r, eval=FALSE}
if (!requireNamespace("BiocManager", quietly=TRUE))
    install.packages("BiocManager")
BiocManager::install("scmeth")
```

Load the package
```{r, warning=FALSE, message=FALSE}
library(scmeth)
```

3. Input files
---------------------
<p style="text-align: justify;">
Main input for most of the function is a *bsseq* object. In the FireCloud 
implementation it is stored as hdf5 file which can be read via 
*loadHDF5SummarizedExperiment* function in *HDF5Array* package.
Code chunk below shows how it can be loaded.
</p>

```{r, eval=FALSE}
directory <- system.file("extdata", "bismark_data", package='scmeth')
bsObject <- HDF5Array::loadHDF5SummarizedExperiment(directory)
```



4. Usage
---------------------

4.1 Report
--------------

<p style="text-align: justify;">
A comprehensive quality control report can be generated in the package via 
**report** function. report function takes the bs object, 
the directory where the report should be saved, organism that this data is 
obtained from and the genomic build. Following is an example usage of the 
**report** function.
</p>

<p style="text-align: justify;">
**report** function also takes two optional arguments: *subSample* and 
*offset*. With subsample parameter users can choose how many CpGs to consider
in the analysis out of all the CpG information available. Our analysis 
have shown that conducting analysis with one million CpGs is suffice to capture
the quality of the samples. However when more CpGs are added they reflect
the full data precisely. Offset parameter is to avoid any telomere region
when subsetting from the beginning of the data. Hence Offset parameter would
avoid first n number of CpGs.
</p>

```{r, eval=FALSE}
report(bsObject, '~/Documents', Hsapiens, "hg38")
```


<p style="text-align: justify;">
Command above generated an html report named *qcReport.html*. It will be stored
in the indicated directory. 
</p>

--------------------------------------------------------------------------------

4.2 Functions
-----------------
<p style="text-align: justify;">
This section will elaborate on some of the main functions and show the usage 
of these functions based on a sample data set that comes along with the 
package.
</p>

<p style="text-align: justify;">
**scmeth** package contains several functions to assess different metrics and 
success of the sequencing process. 
</p>
### coverage
<p style="text-align: justify;">
One main metric is the CpG coverage. Coverage of the CpG can be assessed in 
different ways. Very basic one is to check how many CpG were observed in each 
sample. **coverage** function can be used to get this information. 
</p>

Loading the data
```{r,  warning=FALSE, message=FALSE, comment=FALSE}
directory <- system.file("extdata", "bismark_data", package='scmeth')
bsObject <- HDF5Array::loadHDF5SummarizedExperiment(directory)
```

```{r}
scmeth::coverage(bsObject)
```

### readmetrics
Read information is important to assess whether sequencing and alignment 
succeeded. **readmetrics** function outputs a visualization showing number
of reads seen in each samples and of those reads what proportion of 
them were mapped to the reference genome. 
```{r, fig.width=6, fig.height=3}
readmetrics(bsObject)
```

### repmask
<p style="text-align: justify;">
CpG Islands are characterized by their high GC content, high level of observed 
to expected ratio of CpGs and length over 500 bp. However some repeat regions 
in the genome also fit the same criteria although they are not bona fide CpG 
Island. Therefore it is important to see how many CpGs are observed in the non 
repeat regions of the genome. **repMask** functions provide information on the 
CpG coverage in non repeat regions of the genome. In order to build the repeat 
mask regions of the genome **repmask** function will require the organism and 
the genome build information.
</p>

```{r, warning=FALSE, message=FALSE}
library(BSgenome.Mmusculus.UCSC.mm10)
load(system.file("extdata", 'bsObject.rda', package='scmeth'))
repMask(bs, Mmusculus, "mm10")
```

### Coverage by Chromosome
<p style="text-align: justify;">
There are several other ways the number of CpGs captured can be visualized. 
One of the way is to observe how the CpGs are distributed across different 
chromosomes. ***chromosomeCoverage** outputs CpG coverage by individual 
chromosomes.(Since the example data only contains information in chromosome 1
only the CpGs covered in chromosome 1 is shown.)

</p>

```{r, warning=FALSE}
chromosomeCoverage(bsObject)
```

### featureCoverage
<p style="text-align: justify;">
Another way to observe the distribution of CpGs is to classify them by the 
genomic features they belong. Some of the features are very specific to the CpG
dense regions such as CpG Islands, CpG Shores, CpG Shelves etc. Others are 
general genomic features such as introns, exons, promoters etc. This 
information can be obtained by **featureCoverage** function. In addition to the
bs object this function requires the genomic features of interest and the 
genome build. Each element in the table represents the fraction of CpGs
seen in particular cell in specific region compared to all the CpGs seen in 
that region.
</p>
```{r, warning=FALSE,message=FALSE}
#library(annotatr)
featureList <- c('cpg_islands', 'cpg_shores', 'cpg_shelves')
DT::datatable(featureCoverage(bsObject, features=featureList, "hg38"))
```
</p>



### cpgDensity
<p style="text-align: justify;">
CpGs are not distributed across the genome uniformly. Most of the genome 
contains very low percentage of CpGs except for the CpG dense regions,
i.e. CpG islands. Bisulfite sequencing targets all the CpGs across the genome,
however reduced representation bisulfite sequencing (RRBS) target CpG dense CpG
islands. Therefore CpG density plot will be a great diagnostic to see whether 
the protocol succeeded. In order to calculate the CpG density a window length 
should be specified. By default **cpgDensity** function chooses 1kB regions. 
Therefore CpG density plot can be used to check whether the protocol
specifically targeted CpG dense or CpG sparse regions or whether CpGs were 
obtained uniformly across the regions.
</p>

```{r, warning=FALSE, message=FALSE}
library(BSgenome.Hsapiens.NCBI.GRCh38)
DT::datatable(cpgDensity(bsObject, Hsapiens, windowLength=1000, small=TRUE))
```

### downsample
<p style="text-align: justify;">
In addition to the CpG coverage, methylation data can be assessed via down
sampling analysis, methylation bias plot and methylation distribution. Down 
sampling analysis is a technique to assess whether the sequencing process 
achieved the saturation level in terms of CpG capture. In order to perform 
down sampling analysis CpGs that are covered at least will be sampled via 
binomial random sampling with given probability. At each probability level 
the number of CpGs captured is assessed. If the number of CpG captured attains
a plateau then the sequencing was successful. **downsample** function provides
a matrix of CpG coverage for each sample at various down sampling rates. The
report renders this information into a plot. Downsampling rate ranges from 
0.01 to 1, however users can change the downsampling rates.
</p>

```{r,warning=FALSE}
DT::datatable(scmeth::downsample(bsObject))
```


### mbiasPlot
<p style="text-align: justify;">
Methylation bias plot shows the methylation along the reads. In a high quality 
samples methylation across the read would be more or less a horizontal line. 
However there could be fluctuations in the beginning or the end of the read due
to the quality of the bases. Single cell sequencing samples also can show 
jagged trend in the methylation bias plot due to low read count. Methylation
bias can be assessed via **mbiasPlot** function. This function takes the mbias
file generated from FireCloud pipeline and generates the methylation bias plot.
</p>

```{r, warning=FALSE, message=FALSE, fig.width=6, fig.height=6}

methylationBiasFile <- '2017-04-21_HG23KBCXY_2_AGGCAGAA_TATCTC_pe.M-bias.txt'
mbiasList <- mbiasplot(mbiasFiles=system.file("extdata", methylationBiasFile,
                                         package='scmeth'))

mbiasDf <- do.call(rbind.data.frame, mbiasList)
meanTable <- stats::aggregate(methylation ~ position + read, data=mbiasDf, FUN=mean)
sdTable <- stats::aggregate(methylation ~ position + read, data=mbiasDf, FUN=sd)
seTable <- stats::aggregate(methylation ~ position + read, data=mbiasDf, FUN=function(x){sd(x)/sqrt(length(x))})
sum_mt<-data.frame('position'=meanTable$position,'read'=meanTable$read,
                       'meth'=meanTable$methylation, 'sdMeth'=sdTable$methylation,
                       'seMeth'=seTable$methylation)

sum_mt$upperCI <- sum_mt$meth + (1.96*sum_mt$seMeth)
sum_mt$lowerCI <- sum_mt$meth - (1.96*sum_mt$seMeth)
sum_mt$read_rep <- paste(sum_mt$read, sum_mt$position, sep="_")

g <- ggplot2::ggplot(sum_mt)
g <- g + ggplot2::geom_line(ggplot2::aes_string(x='position', y='meth',
                                                colour='read'))
g <- g + ggplot2::geom_ribbon(ggplot2::aes_string(ymin = 'lowerCI',
                        ymax = 'upperCI', x='position', fill = 'read'),
                        alpha=0.4)
g <- g + ggplot2::ylim(0,100) + ggplot2::ggtitle('Mbias Plot')
g <- g + ggplot2::ylab('methylation')
g

```
</p>

### methylationDist
<p style="text-align: justify;">
**methylationDist** function provides the methylation distribution of the 
samples. In this visualization methylation is divided into quantiles and
ordered according to the cells with the lowest methylation to highest 
methylation. In single cell analysis almost all CpGs will be in the highest
quantile or the lowest quantile. This visualization provides information on 
whether there are cells with intermediate methylation. Ideally , in single
cell methylation most methylation should be either 1 or 0. If there
are large number of intermediate methylation this indicates there might be
some error in sequencing. 
</p>
```{r, warning=FALSE, message=FALSE, fig.width=6, fig.height=3}
methylationDist(bsObject)
```


### bsConversionPlot
<p style="text-align: justify;">
Another important metric in methylation analysis is the bisulfite conversion 
rate. Bisulfite conversion rate indicates out of all the Cytosines in the non
CpG context what fraction of them were methylated. Ideally this number should
be 1 or 100% indicating none of the non CpG context cytosines are methylated. 
However in real data this will not be the case, yet bisulfite conversion
rate below 95% indicates some problem with sample preparation. 
**bsConversionPlot** function generates a plot showing this metric for each 
sample.
</p>
```{r, warning=FALSE, message=FALSE, fig.width=4, fig.height=6}
bsConversionPlot(bsObject)
```

```{r, warning=FALSE, message=FALSE}
sessionInfo()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/repMask.R
\name{repMask}
\alias{repMask}
\title{Provides Coverage metrics in the repeat masker region}
\usage{
repMask(bs, organism, genome)
}
\arguments{
\item{bs}{bsseq object}

\item{organism}{scientific name of the organism of interest,
e.g. Mmusculus or Hsapiens}

\item{genome}{reference alignment, i.e. mm10 or hg38}
}
\value{
Data frame with sample name and coverage in repeat masker regions
}
\description{
Provides Coverage metrics in the repeat masker region
}
\examples{
library(BSgenome.Mmusculus.UCSC.mm10)
library(AnnotationHub)
load(system.file("extdata", 'bsObject.rda', package='scmeth'))
repMask(bs, Mmusculus, 'mm10')
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/coverage.R
\name{coverage}
\alias{coverage}
\title{Coverage for single cells}
\usage{
coverage(bs, subSample = 1e+06, offset = 50000)
}
\arguments{
\item{bs}{bsseq object}

\item{subSample}{number of CpGs to subsample.
Default value is 1000000.}

\item{offset}{how many CpGs to offset when subsampling
Default value is set to be 50000, i.e. first 50000 CpGs will
be ignored in subsampling.}
}
\value{
vector of coverage for the cells in bs object
}
\description{
Provides Coverage for each cell in a library pool
}
\examples{
directory <- system.file("extdata/bismark_data", package='scmeth')
bs <- HDF5Array::loadHDF5SummarizedExperiment(directory)
coverage(bs)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readmetrics.R
\name{readmetrics}
\alias{readmetrics}
\title{Provide graphics for read information}
\usage{
readmetrics(bs)
}
\arguments{
\item{bs}{bsseq object}
}
\value{
Plot showing the mapped and unmapped read information for each cell
}
\description{
Plot the mapped and unmapped reads
}
\examples{
directory <- system.file("extdata/bismark_data", package='scmeth')
bs <- HDF5Array::loadHDF5SummarizedExperiment(directory)
readmetrics(bs)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methylationDist.R
\name{methylationDist}
\alias{methylationDist}
\title{Provide graphics for methylation distribution}
\usage{
methylationDist(bs)
}
\arguments{
\item{bs}{bsseq object}
}
\value{
mean methylation for each sample
}
\description{
Plot the methylation distribution for the cells in bsseq object
}
\examples{
directory <- system.file("extdata/bismark_data", package='scmeth')
bs <- HDF5Array::loadHDF5SummarizedExperiment(directory)
methylationDist(bs)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scmeth.R
\docType{package}
\name{scmeth}
\alias{scmeth}
\alias{scmeth-package}
\title{scmeth: a package to conduct quality control analysis for methylation data.
Most functions can be applied to both bulk and single-cell methylation
while other functions are specific to single-cell methylation data.
scmeth is especially customized to use the output from the FireCloud
implementation of methylation pipeline to produce comprehensive
quality control report}
\description{
scmeth: a package to conduct quality control analysis for methylation data.
Most functions can be applied to both bulk and single-cell methylation
while other functions are specific to single-cell methylation data.
scmeth is especially customized to use the output from the FireCloud
implementation of methylation pipeline to produce comprehensive
quality control report
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chromosomeCoverage.R
\name{chromosomeCoverage}
\alias{chromosomeCoverage}
\title{CpG covergae in each chromosome}
\usage{
chromosomeCoverage(bs)
}
\arguments{
\item{bs}{bsseq object}
}
\value{
matrix of chromsome covergae with
column and rows indicating the samples and the chromosome respectively
}
\description{
Provides Coverage metrics for each sample by the chromosome
}
\examples{
directory <- system.file("extdata/bismark_data",package='scmeth')
bs <- HDF5Array::loadHDF5SummarizedExperiment(directory)
chromosomeCoverage(bs)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mbiasPlot.R
\name{mbiasplot}
\alias{mbiasplot}
\title{Methylation bias plot}
\usage{
mbiasplot(dir = NULL, mbiasFiles = NULL)
}
\arguments{
\item{dir}{directory name with mbias files}

\item{mbiasFiles}{list of mbias files}
}
\value{
Returns a list containing the methylation across the read position in original top
and original bottom strand both in forward and reverse reads for multiple samples
}
\description{
Plot the methylation at each position of the read to observe any biases in
the methylation
based on the read position
}
\examples{
mbiasFile <- '2017-04-21_HG23KBCXY_2_AGGCAGAA_TATCTC_pe.M-bias.txt'
mbiasplot(mbiasFiles=system.file("extdata", mbiasFile, package='scmeth'))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cpgDensity.R
\name{cpgDensity}
\alias{cpgDensity}
\title{Provides Coverage by the CpG density. CpG Density is defined as the number
of CpGs observed in certain base pair long region.}
\usage{
cpgDensity(bs, organism, windowLength = 1000, small = FALSE)
}
\arguments{
\item{bs}{bsseq object}

\item{organism}{scientific name of the organism of interest,
e.g. Mmusculus or Hsapiens}

\item{windowLength}{Length of the window to calculate the density}

\item{small}{Indicator for a small dataset, cpg density is calculated more
memory efficiently for large dataset but for small dataset a different quicker
method is used
Default value for window length is 1000 basepairs.}
}
\value{
Data frame with sample name and coverage in repeat masker regions
}
\description{
Provides Coverage by the CpG density. CpG Density is defined as the number
of CpGs observed in certain base pair long region.
}
\examples{
library(BSgenome.Hsapiens.NCBI.GRCh38)
directory <- system.file("extdata/bismark_data", package='scmeth')
bs <- HDF5Array::loadHDF5SummarizedExperiment(directory)
cpgDensity(bs, Hsapiens, 1000, small=TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/featureCoverage.R
\name{featureCoverage}
\alias{featureCoverage}
\title{Coverage based on the genomic feature}
\usage{
featureCoverage(bs, features, genomebuild)
}
\arguments{
\item{bs}{bsseq object}

\item{features}{list of genomic features, e.g. genes_exons, genes_introns,
cpg_islands, cpg_shelves
Names are based on the annotatr packages, so all the features provided by the
annotatr
package will be supported in this function}

\item{genomebuild}{reference alignment, i.e. mm10 or hg38}
}
\value{
a data frame with genomic feature names and the number of
CpG covered in each feature
}
\description{
Provides Coverage metrics for the sample by each genomic features provided
by the user
}
\examples{
directory <- system.file("extdata/bismark_data", package='scmeth')
bs <- HDF5Array::loadHDF5SummarizedExperiment(directory)
featureCoverage(bs, c('cpg_islands', 'cpg_shores'), 'hg38')
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cpgDiscretization.R
\name{cpgDiscretization}
\alias{cpgDiscretization}
\title{Discretize the CpG methylation values
to align with single cell analysis}
\usage{
cpgDiscretization(bs, subSample = 1e+06, offset = 50000,
  coverageVec = NULL)
}
\arguments{
\item{bs}{bsseq object}

\item{subSample}{number of CpGs to subsample.
Default value is 1000000.}

\item{offset}{how many CpGs to offset when subsampling
Default value is set to be 50000, i.e. first 50000 CpGs will
be ignored in subsampling.}

\item{coverageVec}{If coverage vector is already calculated provide it to
speed up the process}
}
\value{
meth discretized methylation matrix

discard total number of removed CpGs from each sample

Percentage of CpGs discarded compared to the total number of CpGs
}
\description{
In single cell analysis overwhelmingly large number of CpGs have binary
methylation
Due to errors in sequencing and amplification many CpGs tend to have
non-binary methylation. Hence
this function catergorizes the non-binary CpGs as methylated if the
methyation is above 0.8 and
unmethylated if the methylation is below 0.2
}
\examples{
directory <- system.file("extdata/bismark_data", package='scmeth')
bs <- HDF5Array::loadHDF5SummarizedExperiment(directory)
cpgDiscretization(bs)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/report.R
\name{report}
\alias{report}
\title{Generates an inclusive report on methylation analysis}
\usage{
report(bsObj, outdirectory, organism, genome, mbiasDir = NULL,
  subSample = 1e+06, offset = 50000, small = FALSE)
}
\arguments{
\item{bsObj}{bsseq object}

\item{outdirectory}{name of the output directory where the
report will be saved}

\item{organism}{scientific name of the organism of interest,
e.g. Mmusculus or Hsapiens}

\item{genome}{reference alignment, e.g. mm10 or hg38
the report will have graphics on read information}

\item{mbiasDir}{Optional argument to provide directory name
that has the mbias files or the list of mbias files}

\item{subSample}{number of CpGs to subsample
Default value is 1000000.}

\item{offset}{how many CpGs to offset when subsampling
Default value is set to be 50000, i.e. first 50000 CpGs will
be ignored in subsampling.}

\item{small}{Indicator for a small dataset, cpg density is calculated more}
}
\value{
Report will be an html file
}
\description{
This function uses most of the functions in this package to
generate a report for the user
}
\examples{
library(BSgenome.Hsapiens.NCBI.GRCh38)
directory <- system.file("extdata/bismark_data", package='scmeth')
bs <- HDF5Array::loadHDF5SummarizedExperiment(directory)
mbiasDirectory=system.file("extdata", package='scmeth')
outDir <- system.file(package='scmeth')
report(bs, outDir, Hsapiens, 'hg38', mbiasDir=mbiasDirectory, small=TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/downsample.R
\name{downsample}
\alias{downsample}
\title{Downsample analysis}
\usage{
downsample(bs, dsRates = c(0.01, 0.02, 0.05, seq(0.1, 0.9, 0.1)),
  subSample = 1e+06, offset = 50000)
}
\arguments{
\item{bs}{bsseq object}

\item{dsRates}{downsampling rate. i.e. the probabaility of sampling
a single CpG
default is list of probabilities ranging from 0.01 to 1
0.01 0.02 0.05 0.10 0.20 0.30 0.40 0.50 0.60 0.70 0.80 0.90
For more continuous saturation curve dsRates can be changed to add more
sampling rates}

\item{subSample}{number of CpGs to subsample
Default value is 1000000.}

\item{offset}{how many CpGs to offset when subsampling
Default value is set to be 50000, i.e. first 50000 CpGs will
be ignored in subsampling.}
}
\value{
Data frame with the CpG coverage for each sample at each
sampling rate
}
\description{
Downsample the CpG coverage matrix for saturation analysis
}
\examples{
directory <- system.file("extdata/bismark_data", package='scmeth')
bs <- HDF5Array::loadHDF5SummarizedExperiment(directory)
scmeth::downsample(bs)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bsConversionPlot.R
\name{bsConversionPlot}
\alias{bsConversionPlot}
\title{Bisulfite conversion rate visualization}
\usage{
bsConversionPlot(bs)
}
\arguments{
\item{bs}{bsseq object}
}
\value{
Plot showing bisulfite conversion rate for each sample
}
\description{
Plot the bisulfite conversion rate for each sample
based on the pheno data in the bs object
}
\examples{
directory <- system.file("extdata/bismark_data", package='scmeth')
bs <- HDF5Array::loadHDF5SummarizedExperiment(directory)
bsConversionPlot(bs)
}
